package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.cloud.base.service.IBasePowerLineCloudService;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineOutput;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.data.GetAllOuListByOuIdInput;
import pps.cloud.system.service.data.GetSysOrganizationViewOutput;
import pps.core.prediction.entity.EnergyConsumptionAnalysisEnt;
import pps.core.prediction.entity.EnergyConsumptionAnalysisView;
import pps.core.prediction.enums.BusinessError;
import pps.core.prediction.mapper.EnergyConsumptionAnalysisMapper;
import pps.core.prediction.mapper.EnergyConsumptionAnalysisViewMapper;
import pps.core.prediction.service.data.energy_consumption_analysis.*;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 能耗分析模块(测试用)
 * todo: 模拟测试用,后续替换
 *
 * @author ZWT
 * @date 2023/11/23
 */
@XText("能耗分析模块")
@XService
public class EnergyConsumptionAnalysisService {

    /**
     * 能耗分析--能耗概览
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetEnergyConsumptionAnalysisViewOutput}>
     */
    @XText("能耗分析--能耗概览")
    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetEnergyConsumptionAnalysisViewOutput> getEnergyConsumptionAnalysisView(XContext context, GetEnergyConsumptionAnalysisViewInput input) {
        Date endTime, startTime;
        //近7天:昨天开始前七天;近30天:昨天开始前30天;上期:所选周期往前推一个周期
        switch (input.getDateType()) {
            case 1:
                endTime = DateUtil.beginOfDay(DateUtil.yesterday());
                startTime = DateUtil.offsetDay(endTime, -6);
                break;
            case 2:
                endTime = DateUtil.beginOfDay(DateUtil.yesterday());
                startTime = DateUtil.offsetDay(endTime, -29);
                break;
            case 3:
                if (ObjectUtil.isNull(input.getEndTime()) || ObjectUtil.isNull(input.getStartTime())) {
                    throw new XServiceException(BusinessError.TheStartEndDateIsMissing);
                }
                endTime = input.getEndTime();
                startTime = input.getStartTime();
                break;
            default:
                throw new XServiceException(BusinessError.DateTypeError);
        }
        GetEnergyConsumptionAnalysisViewOutput output = new GetEnergyConsumptionAnalysisViewOutput();
        List<DynamicQueryBasePowerLineOutput> lineList = this.getBasePowerLineListByOuIdList(context, this.getOrgIdsByPath(context, input.getOuId()));
        if (CollUtil.isEmpty(lineList)) {
            output.setPowerGeneration(BigDecimal.ZERO);
            output.setPowerConsumption(BigDecimal.ZERO);
            output.setAbsorptionRate(BigDecimal.ZERO);
        } else {
            List<String> lineIdList = lineList.stream()
                    .map(DynamicQueryBasePowerLineOutput::getId)
                    .collect(Collectors.toList());
            EnergyConsumptionAnalysisMapper mapper = context.getBean(EnergyConsumptionAnalysisMapper.class);
            EnergyConsumptionAnalysisEnt analysisEnt = mapper.selectOne(new QueryWrapper<EnergyConsumptionAnalysisEnt>()
                    .select("IFNULL( SUM( power_generation ), 0 ) AS power_generation",
                            "IFNULL( SUM( power_consumption ), 0 ) AS power_consumption")
                    .lambda()
                    .in(EnergyConsumptionAnalysisEnt::getLineId, lineIdList)
                    .between(EnergyConsumptionAnalysisEnt::getDataDate, startTime, endTime)
            );
            output.setPowerGeneration(analysisEnt.getPowerGeneration());
            output.setPowerConsumption(analysisEnt.getPowerConsumption());
            //消纳率:光伏发电/累计用电*100%
            if (analysisEnt.getPowerConsumption().compareTo(BigDecimal.ZERO) > 0) {
                output.setAbsorptionRate(analysisEnt.getPowerGeneration().divide(analysisEnt.getPowerConsumption(), 4, RoundingMode.HALF_UP)
                        .multiply(new BigDecimal(100)));
            } else {
                output.setAbsorptionRate(BigDecimal.ZERO);
            }
        }
        return XSingleResult.success(output);
    }

    /**
     * 能耗分析--绿电消纳/用电趋势
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryEnergyConsumptionAnalysisViewOutput}>
     */
    @XText("能耗分析--绿电消纳/用电趋势")
    @XApiAnonymous
    @XApiGet
    public XListResult<DynamicQueryEnergyConsumptionAnalysisViewOutput> queryEnergyConsumptionAnalysisList(XContext context, DynamicQueryEnergyConsumptionAnalysisInput input) {
        EnergyConsumptionAnalysisMapper mapper = context.getBean(EnergyConsumptionAnalysisMapper.class);
        return XListResult.success();
    }


    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryEnergyConsumptionAnalysisViewOutput> queryEnergyConsumptionAnalysisView(XContext context, QueryEnergyConsumptionAnalysisViewInput input) {
        EnergyConsumptionAnalysisViewMapper mapper = context.getBean(EnergyConsumptionAnalysisViewMapper.class);
        EnergyConsumptionAnalysisView record = new EnergyConsumptionAnalysisView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<EnergyConsumptionAnalysisView> list = mapper.selectList(record);
        PageInfo<EnergyConsumptionAnalysisView> pageInfo = new PageInfo<>(list);
        List<QueryEnergyConsumptionAnalysisViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryEnergyConsumptionAnalysisViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 获取所有path包含组织ID的组织
     *
     * @param context 上下文
     * @param ouId    ou id
     * @return {@link List}<{@link String}>
     */
    private List<String> getOrgIdsByPath(XContext context, String ouId) {
        SysOrganizationCloudService organizationCloudService = context.getBean(SysOrganizationCloudService.class);
        GetAllOuListByOuIdInput ouIdInput = new GetAllOuListByOuIdInput();
        ouIdInput.setOuId(ouId);
        XListResult<GetSysOrganizationViewOutput> allListByOuId = organizationCloudService.getAllListByOuId(context, ouIdInput);
        allListByOuId.throwIfFail();
        return allListByOuId.getResult().stream()
                .map(GetSysOrganizationViewOutput::getId)
                .collect(Collectors.toList());
    }

    /**
     * 查询组织列表下所有线路
     *
     * @param context  上下文
     * @param ouIdList ou-id列表
     * @return {@link List}<{@link DynamicQueryBasePowerLineOutput}>
     */
    private List<DynamicQueryBasePowerLineOutput> getBasePowerLineListByOuIdList(XContext context, List<String> ouIdList) {
        IBasePowerLineCloudService basePowerLineCloudService = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineOutput> lineList = basePowerLineCloudService.getBasePowerLineListByOuIdList(context, DynamicQueryBasePowerLineInput.builder()
                .ouIdList(ouIdList)
                .build());
        lineList.throwIfFail();
        return lineList.getResult();
    }
}
